/*
 * aKEY -- registration key system
 *
 * Demo of registration key generation
 *
 * Copyright (c) 2000-2002 by Joergen Ibsen / Jibz
 * All Rights Reserved
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>

#include "keygen.h"

/* compiler specific stuff */
#ifdef aKEY_UNIX_STYLE
   #include <unistd.h>
#else
   #include <io.h>
#endif

#ifndef O_BINARY
   #define O_BINARY 0
#endif

/* global variables used */
int keyfile = -1;

const char keystart[] =
  "--- BEGIN REGISTRATION KEY ---\r\n"
  "Registration key for aKEY Demo. Do not distribute or modify!\r\n\r\n";

const char keyend[] =
  "--- END REGISTRATION KEY ---\r\n";

/* ======================================================================= *
 *  FUNCTIONS                                                              *
 * ======================================================================= */

void aKEY_demo_error(char *why)
{
  printf("\n\n\007ERR: %s\n", why);

  if (keyfile != -1) close(keyfile);

  exit(-1);
}

/* ======================================================================= *
 *  MAIN                                                                   *
 * ======================================================================= */

int main(int argc, char *argv[])
{
   void *key;
   char *keyout;
   char regname[64];
   int i;

   printf("===============================================================================\n"
          "aKEY key-generation demo        Copyright (c) 2000-2002 by Joergen Ibsen / Jibz\n"
          "                                                            All Rights Reserved\n"
          "===============================================================================\n\n");

   /* write syntax when not enough parameters */
   if (argc < 2)
   {
      printf("  Syntax:   aKEYgen <key file>\n\n");
      return(1);
   }

   /* get registration name */
   printf("- please enter registration name (minimum 4 characters)\n");
   printf("  : ");
   fgets(regname, sizeof(regname) - 1, stdin);
   for (i = strlen(regname); (i > 0) && (regname[i] < ' '); --i) regname[i] = '\0';

   printf("- creating key\n");

   {
      clock_t clock_start, clock_end;

      clock_start = clock();

      if ((key = aKEY_create_key(512, time(NULL))) == NULL)
         aKEY_demo_error("key creation error");

      aKEY_add_block(key, BLOCK_USER_DATA, regname, strlen(regname));

      aKEY_sign_key(key);

      keyout = aKEY_encode_key(key);

      clock_end = clock();
      printf("- key creation took %.2f seconds\n", (double)(clock_end - clock_start) / (double)CLOCKS_PER_SEC);
   }

   printf("- writing key\n");

   /* open key file */
   if ((keyfile = open(argv[1], O_WRONLY | O_CREAT | O_BINARY | O_TRUNC, S_IREAD | S_IWRITE)) == -1)
      aKEY_demo_error("unable to open key file for writing");

   /* write encoded key to file */
   write(keyfile, keystart, sizeof(keystart) - 1);
   write(keyfile, keyout, strlen(keyout));
   write(keyfile, keyend, sizeof(keyend) - 1);

   close(keyfile);

   /* free memory */
   aKEY_destroy_key(key);

   return(0);
}

/* ======================================================================= *
 *  END                                                                    *
 * ======================================================================= */
