/*
 * aKEY -- registration key system
 *
 * Demo of registration key test from file
 *
 * Copyright (c) 2000-2002 by Joergen Ibsen / Jibz
 * All Rights Reserved
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>

#include "keycheck.h"

/* compiler specific stuff */
#ifdef aKEY_UNIX_STYLE
   #include <unistd.h>
#else
   #include <io.h>
#endif

#ifndef O_BINARY
   #define O_BINARY 0
#endif

/* global variables used */
int keyfile = -1;
char *encoded_key = NULL;
char *key = NULL;

/* ======================================================================= *
 *  FUNCTIONS                                                              *
 * ======================================================================= */

void aKEY_demo_error(char *why)
{
  printf("\n\n\007ERR: %s\n", why);

  if (encoded_key != NULL) free(encoded_key);
  if (key != NULL) free(key);
  if (keyfile != -1) close(keyfile);

  exit(-1);
}

/* ======================================================================= *
 *  MAIN                                                                   *
 * ======================================================================= */

int main(int argc, char *argv[])
{
   int dsize, res, keyfilesize;
   char *udata;

   printf("===============================================================================\n"
          "aKEY key-checking demo          Copyright (c) 2000-2002 by Joergen Ibsen / Jibz\n"
          "                                                            All Rights Reserved\n"
          "===============================================================================\n\n");

   /* write syntax when not enough parameters */
   if (argc < 2)
   {
      printf("  Syntax:   aKEYtest <key file>\n\n");
      return(1);
   }

   printf("- reading key data\n");

   /* open key file */
   if ((keyfile = open(argv[1], O_RDONLY | O_BINARY)) == -1)
      aKEY_demo_error("unable to open key file for reading");

   /* get size of key file */
   keyfilesize = lseek(keyfile, 0, SEEK_END);
   if (keyfilesize < 64) aKEY_demo_error("key file is too small");

   /* allocate memory */
   if ((encoded_key = (char *) malloc(keyfilesize)) == NULL) aKEY_demo_error("not enough memory");
   if ((key = (char *) malloc(keyfilesize)) == NULL) aKEY_demo_error("not enough memory");

   /* read key into buffer */
   lseek(keyfile, 0, SEEK_SET);
   read(keyfile, encoded_key, keyfilesize);

   close(keyfile);

   printf("- key file read, size %d\n", keyfilesize);

   udata = 0;

   /* decode key from encoded_key[] to key[] */
   dsize = aKEY_decode_key(encoded_key, key, keyfilesize);

   if (dsize)
   {
      printf("- key decoded, %d bytes\n", dsize);

      {
         clock_t clock_start, clock_end;

         clock_start = clock();

         /* check if key is valid */
         res = aKEY_check_key(key, dsize);

         clock_end = clock();
         printf("- decryption took %.2f seconds\n", (double)(clock_end - clock_start) / (double)CLOCKS_PER_SEC);
      }

      if (res)
      {
         printf("- key verified\n");

         /* get a pointer to the user data */
         udata = aKEY_get_user_data(key);
      }
   }

   /* here, key was valid if udata != NULL */
   if (udata != NULL)
   {
      printf("- registered to: %s\n", udata);
   } else {
      printf("- not registered\n");
   }

   /* free memory */
   if (encoded_key != NULL) free(encoded_key);
   if (key != NULL) free(key);

   return(0);
}

/* ======================================================================= *
 *  END                                                                    *
 * ======================================================================= */
